@@ -433,6 +433,81 @@ class TestUserDefiniedModel(CommonTests, unittest.TestCase):
         self.assertEqual(param_values['p1_amplitude'], 1)
         self.assertEqual(param_values['p2_amplitude'], 2)
 
+    def test_hints_for_peakmodels(self):
+        # test that height/fwhm do not cause asteval errors.
+
+        x = np.linspace(-10, 10, 101)
+        y = np.sin(x / 3)  + x /100.
+
+        m1 = models.LinearModel(prefix='m1_')
+
+        params = m1.guess(y, x=x)
+
+        m2 = models.GaussianModel(prefix='m2_')
+        params.update(m2.make_params())
+
+        m = m1 + m2
+
+        param_values = {name: p.value for name, p in params.items()}
+        self.assertTrue(param_values['m1_intercept'] < -0.0)
+        self.assertEqual(param_values['m2_amplitude'], 1)
+
+    def test_weird_param_hints(self):
+        # tests Github Issue 312, a very weird way to access param_hints
+        def func(x, amp):
+            return amp*x
+
+        m = Model(func)
+        models = {}
+        for i in range(2):
+            m.set_param_hint('amp', value=1)
+            m.set_param_hint('amp', value=25)
+
+            models[i] = Model(func, prefix='mod%i_' % i)
+            models[i].param_hints['amp'] = m.param_hints['amp']
+
+        self.assertEqual(models[0].param_hints['amp'],
+                         models[1].param_hints['amp'])
+
+
+    def test_composite_model_with_expr_constrains(self):
+        """Smoke test for composite model fitting with expr constraints.
+        """
+        y = [  0,   0,   4,   2,   1,   8,  21,  21,  23,  35,  50,  54,  46,
+             70,   77,  87,  98, 113, 148, 136, 185, 195, 194, 168, 170, 139,
+             155, 115, 132, 109, 102,  85,  69,  81,  82,  80,  71,  64,  79,
+             88,  111,  97,  97,  73,  72,  62,  41,  30,  13,   3,   9,   7,
+             0,     0,   0]
+        x = np.arange(-0.2, 1.2, 0.025)[:-1] + 0.5*0.025
+
+        def gauss(x, sigma, mu, A):
+            return A*np.exp(-(x-mu)**2/(2*sigma**2))
+
+        # Initial values
+        p1_mu = 0.2
+        p1_sigma = 0.1
+        #p2_mu = 0.8
+        p2_sigma = 0.1
+
+        peak1 = Model(gauss, prefix='p1_')
+        peak2 = Model(gauss, prefix='p2_')
+        model = peak1 + peak2
+
+        model.set_param_hint('p1_mu', value=p1_mu, min=-1, max=2)
+        #model.set_param_hint('p2_mu', value=p2_mu, min=-1, max=2)
+        model.set_param_hint('p1_sigma', value=p1_sigma, min=0.01, max=0.2)
+        model.set_param_hint('p2_sigma', value=p2_sigma, min=0.01, max=0.2)
+        model.set_param_hint('p1_A', value=100, min=0.01)
+        model.set_param_hint('p2_A', value=50, min=0.01)
+
+        # Constrains the distance between peaks to be > 0
+        model.set_param_hint('pos_delta', value=0.3, min=0)
+        model.set_param_hint('p2_mu', min=-1, expr='p1_mu + pos_delta')
+
+        # Test fitting
+        result = model.fit(y, x=x)
+        self.assertTrue(result.params['pos_delta'].value > 0)
+
 
 class TestLinear(CommonTests, unittest.TestCase):
 
@@ -475,7 +550,6 @@ class TestPolynomialOrder3(CommonTests, unittest.TestCase):
 
 
 class TestConstant(CommonTests, unittest.TestCase):
-
     def setUp(self):
         self.true_values = lambda: dict(c=5)
         self.guess = lambda: dict(c=2)
@@ -486,7 +560,6 @@ class TestConstant(CommonTests, unittest.TestCase):
         raise nose.SkipTest("ConstantModel has not independent_vars.")
 
 class TestPowerlaw(CommonTests, unittest.TestCase):
-
     def setUp(self):
         self.true_values = lambda: dict(amplitude=5, exponent=3)
         self.guess = lambda: dict(amplitude=2, exponent=8)
@@ -495,9 +568,18 @@ class TestPowerlaw(CommonTests, unittest.TestCase):
 
 
 class TestExponential(CommonTests, unittest.TestCase):
-
     def setUp(self):
         self.true_values = lambda: dict(amplitude=5, decay=3)
         self.guess = lambda: dict(amplitude=2, decay=8)
         self.model_constructor = models.ExponentialModel
         super(TestExponential, self).setUp()
+
+
+class TestComplexConstant(CommonTests, unittest.TestCase):
+    def setUp(self):
+        self.true_values = lambda: dict(re=5,im=5)
+        self.guess = lambda: dict(re=2,im=2)
+        self.model_constructor = models.ComplexConstantModel
+        super(TestComplexConstant, self).setUp()
+
+#
